@@ -2,8 +2,7 @@ import tensorflow as tf
 import numpy as np
 import enum
 import re
-import random
-from collections import namedtuple
+from collections import namedtuple, OrderedDict
 from .error import SchemaError
 from . import fgraph
 
@@ -62,7 +61,7 @@ def bcast_dim(dims, comp):
         else:
             return dims[comp]
 
-def range_under_size(idx_ranges, max_prod):
+def range_under_size(idx_ranges, max_prod, rng):
     """
     generate a tuple of dims between idx_ranges, with prod() <= total
     each element of idx_ranges represents a component of an index.
@@ -89,7 +88,7 @@ def range_under_size(idx_ranges, max_prod):
             ub = 1e10
         else:
             ub = max_prod // used
-        d = random.randint(lo, min(hi, ub))
+        d = rng.randint(lo, min(hi, ub))
         cumul *= d
         dims.append(d)
     return dims
@@ -177,15 +176,29 @@ class ShapeEdit(object):
                 return off, off + rank 
             off += rank
 
-    def get_input_dims(self):
+    def get_input_dims(self, use_scalars=False):
         # return the imputed index dims, or raise an error if ambiguous
+        # if `use_scalars`, convert any length 1 dims to scalar if the
+        # index has a constant rank of 1
         if any(len(usage) > 1 for usage in self.usage_map.values()):
             raise RuntimeError(
                 f'{type(self).__qualname__}: ambiguous index usage')
         index_dims = {}
         for idx, usage in self.usage_map.items():
             assert len(usage) == 1, 'Index Usage should be length 1'
-            index_dims[idx] = next(iter(usage))
+            dims = next(iter(usage))
+            if isinstance(dims, int):
+                index_dims[idx] = dims
+                continue
+
+            if use_scalars and self.op.index[idx].fixed_rank():
+                if len(dims) != 1:
+                    raise RuntimeError(
+                        f'Index {idx} is fixed rank = 1 but found usage of '
+                        f'non-rank-1')
+                index_dims[idx] = dims[0]
+            else:
+                index_dims[idx] = dims
         return index_dims
 
     def maybe_get_index_dim(self, idx):
@@ -479,7 +492,9 @@ class DataFormats(object):
             self.formats = { SINGLE_FORMAT: (0, None) }
             self.rank_index = None
         else:
-            self.formats = formats
+            key_func = lambda t: '___' if t is None else t[0]
+            sorted_keys = sorted(formats.keys(), key=key_func)
+            self.formats = OrderedDict({k: formats[k] for k in sorted_keys})
             self.rank_index = rank_index
 
     def single(self):
@@ -515,20 +530,20 @@ class DataFormats(object):
         Return the data_format corresponding to the layout and rank
         combination.
         """
-        it = self.formats.items()
+        items = self.formats.items()
         rank = None if self.rank_index is None else ranks[self.rank_index]
         if rank is None:
-            return next((df for df, (l, _) in it if l == layout), None)
+            return next((df for df, (l, _) in items if l == layout), None)
         else:
-            return next((df for df, (l, r) in it if l == layout and (r is None
+            return next((df for df, (l, r) in items if l == layout and (r is None
                 or r == rank)), None)
 
     def layout_formats(self, layout):
         """
         Return all formats for a particular layout, in rank order
         """
-        it = self.formats.items()
-        fmts = [ (df, r) for df, (l, r) in it if l == layout and df is not None]
+        items = self.formats.items()
+        fmts = [ (df, r) for df, (l, r) in items if l == layout and df is not None]
         fmts = sorted(fmts, key=lambda tup: tup[1])
         return [ df for df, _ in fmts ]
 
@@ -553,17 +568,22 @@ class DataFormats(object):
         return self.formats[data_format][1]
 
 class IndexPredicate(object):
-    def __init__(self, name, cwise, pfunc, pfunc_t, indices):
+    def __init__(self, name, cwise, pfunc, pfunc_t, indices, *arg_names):
         self.name = name
         self.cwise = cwise
         self.pfunc = pfunc
         self.pfunc_t = pfunc_t
         self.indices = indices
+        self.arg_names = arg_names
 
-    def __call__(self, *dims):
+    def __call__(self, *dims_argvals):
         """
         Evaluate the predicate either component-wise or all at once
         """
+        ninds = len(self.indices)
+        dims = dims_argvals[:ninds]
+        argvals = dims_argvals[ninds:]
+
         if self.cwise:
             ranks = { len(d) for d in dims if isinstance(d, (list, tuple)) }
             if len(ranks) > 1:
@@ -571,17 +591,17 @@ class IndexPredicate(object):
                     f'IndexPredicate is component-wise but input indices '
                     f'{self.indices} have different ranks')
             elif len(ranks) == 0:
-                return self.pfunc(*dims)
+                return self.pfunc(*dims, *argvals)
             else:
                 rank = ranks.pop()
                 for c in range(rank):
                     ins = tuple(d if isinstance(d, int) else d[c] for d in dims)
-                    if not self.pfunc(*ins):
+                    if not self.pfunc(*ins, *argvals):
                         return False
                 return True
         else:
             try:
-                return self.pfunc(*dims)
+                return self.pfunc(*dims, *argvals)
             except BaseException as ex:
                 raise SchemaError(f'IndexPredicate error in non-cw mode: {ex}')
     
