@@ -17,6 +17,12 @@ def filter_pad(filt, dilation):
 def filter_pad_t(filt, dilation):
     return f'({filt} - 1) * {dilation} + 1'
 
+def dilate(s, d):
+    return (s - 1) * d + 1
+
+def dilate_t(s,  d):
+    return f'({s} - 1) * {d} + 1'
+
 def ceildiv(a, b):
     return math.ceil(a / b)
 
@@ -24,7 +30,7 @@ def mod(a, b):
     return a % b
 
 def reduce_prod(a):
-    return np.prod(a)
+    return int(np.prod(a))
 
 def conv(input, pad_filter, stride, padding):
     if padding == 'VALID':
