@@ -1,12 +1,13 @@
 from opschema import genlib
-from opschema.complib import filter_pad, filter_pad_t, tconv, tconv_t
+from opschema.complib import dilate, dilate_t, tconv, tconv_t
 
 def init_schema(op):
-    op.add_index('b', 'batch', (1, 10))
+    op.add_index('b', 'batch', (1, 1))
     op.add_index('i', 'input spatial', (1,3))
     op.add_index('k', 'input channel', 1)
+    op.add_index('j', 'strided input spatial', 'i')
     op.add_index('f', 'filter spatial', 'i')
-    op.add_index('p', 'pad filter spatial', 'i')
+    op.add_index('g', 'dilated filter spatial', 'i')
     op.add_index('s', 'strides', 'i')
     op.add_index('d', 'dilations', 'i')
     op.add_index('l', 'output channel', 1)
@@ -29,8 +30,33 @@ def init_schema(op):
     op.gen_dims('k', 30)
     op.gen_dims('l', 30)
 
-    op.comp_dims_cw('p', filter_pad, filter_pad_t, 'fd')
-    op.comp_dims_cw('o', tconv, tconv_t, 'ips', 'padding')
+    jdims =   lambda i, s: i * s
+    jdims_t = lambda i, s: f'{i} * {s}'
+
+    # input is dilated with 'strides' 
+    op.comp_dims_cw('j', dilate, dilate_t, 'is') 
+    # filter is dilated with 'dilations'
+    op.comp_dims_cw('g', dilate, dilate_t, 'fd')
+
+    def odims_gen(rng, j, g):
+        val = j + g - 1
+        yield val, val
+
+    op.gen_dims_func('o', odims_gen, 'jg', 1000, False)   
+
+    def oi_pred(o, j, g, padding):
+        if padding == 'VALID':
+            return (o - g + 1) == j
+        else:
+            return o == j
+
+    def oi_pred_t(o, j, g, padding):
+        if padding == 'VALID':
+            return f'({o} - {g} + 1) must equal {j}'
+        else:
+            return f'{o} must equal {j}'
+
+    op.dims_pred_cw('in-out-pred', oi_pred, oi_pred_t, 'ojg', 'padding')
 
     op.valid_dtypes('input', ('float',))
     op.equate_dtypes('filters', 'input')
