@@ -1,10 +1,11 @@
 import tensorflow as tf
 import traceback
 import inspect
+from collections import OrderedDict
 import sys, io, os
 import re
 import itertools
-from random import choice
+from random import Random
 from . import predicates as pr
 from . import generators as ge
 from . import infer as nf
@@ -109,10 +110,13 @@ class OpSchema(object):
 
         # Graphs
         self.pred_graph = {}
-        self.gen_graph = {} # idx => node, for generating inventory
+        self.gen_graph = {} 
         self.inf_graph = {}
         self.dims_graph = {}
 
+        # Random Number Generators
+        self.gen_rng = Random()
+
         # provides information for gr.DimsInput nodes
         self.dims_graph_input = {}
 
@@ -455,7 +459,7 @@ class OpSchema(object):
         """
         comp_nodes = self._comp_dims_nodes() 
         if len(comp_nodes) == 0:
-            return 'None'
+            return None
 
         render = base.RenderCompDims(self)
         inp_nodes = self._dims_input_nodes()
@@ -573,8 +577,8 @@ class OpSchema(object):
         index_inv = self.index_inventory()
         signature = self.signature_report()
         index_ranks = self.index_ranks_report()
-        computed_dims_snake = self.comp_dims_report(True)
-        computed_dims_olc = self.comp_dims_report(False)
+        comp_dims_snake = self.comp_dims_report(True)
+        comp_dims_olc = self.comp_dims_report(False)
         dtype_rules = self.dtype_rules_report()
         combo_rules = self.excluded_dtypes_report()
 
@@ -585,8 +589,10 @@ class OpSchema(object):
         finals.append(f'Signatures\n\n{signature}')
         finals.append(f'Index ranks\n\n{index_ranks}')
 
-        if computed_dims_snake is not None:
-            finals.append(f'Computed dimensions\n\n{computed_dims_snake}\n\n{computed_dims_olc}')
+        if comp_dims_snake is None:
+            finals.append(f'Computed dimensions\n\nNone')
+        else:
+            finals.append(f'Computed dimensions\n\n{comp_dims_snake}\n\n{comp_dims_olc}')
 
         finals.append(f'DType Rules\n\n{dtype_rules}')
 
@@ -778,38 +784,12 @@ class OpSchema(object):
     def _prep_gen_tests(self):
         self.avail_test_edits = 1
 
-    def generate_args(self):
+    def generate_args(self, rand_seed=12345):
         self._prep_gen_tests()
-
-        live_kinds = (
-                ge.DTypeEquiv, ge.DTypeIndiv, ge.DTypesNotImpl,
-                ge.IndexRanks, ge.RankRange, ge.RankEquiv,
-                ge.Layout, ge.Sig, ge.SigMap,
-                ge.ArgRanks,
-                ge.ArgIndels,
-                ge.Options,
-                ge.ArgMutations,
-                )
-        out_kinds = (
-                ge.DTypesNotImpl,
-                # ge.IndexRanks,
-                # ge.SigMap,
-                # ge.ArgRanks,
-                ge.Options,
-                ge.ArgIndels,
-                ge.ArgMutations,
-                )
-        nodes = list(self.gen_graph.values())
-        live = [n for n in nodes if isinstance(n.func, live_kinds)]
-        out = [n for n in nodes if isinstance(n.func, out_kinds)] 
-        live = nodes
+        live = self.gen_graph.values()
         out = [self._gen_node(ge.Args)]
-
-        ctr = 1
-        for op_args in fgraph.gen_graph_values(live, out):
-            # print(f'\r{ctr}', end='', flush=True)
-            # ctr += 1
-            # print(op_args)
+        self.gen_rng.seed(rand_seed)
+        for op_args in fgraph.gen_graph_values(live, out, self):
             yield op_args[0] # extract tuple element
 
     def validate(self, out_dir, test_ids, skip_ids, dtype_err_quota):
@@ -826,6 +806,7 @@ class OpSchema(object):
         stats = { k: 0 for k in cats }
 
         op_args_gen = self.generate_args()
+        # op_args_gen = list(self.generate_args())
         # print(f'Generated {len(op_args_list)} test cases')
 
         for test_id, op_args in enumerate(op_args_gen, 1):
@@ -840,9 +821,10 @@ class OpSchema(object):
                 else:
                     test_ids.remove(test_id)
 
-            # print(op_args)
             # self.show_graph_calls = True
             arg_dict = { k: v.value() for k, v in op_args.items() }
+            # print(test_id, op_args)
+            # continue
             string_err = io.BytesIO()
             try:
                 with stderr_redirector(string_err):
@@ -1067,13 +1049,18 @@ class OpSchema(object):
 
     def gen_dims_func(self, out_sig, func, in_sig, max_prod, do_scalar, *pars):
         """
-        Calls func(*in_dims, *pars), which yields one or more
+        Calls func(gen_rng, *in_dims, *pars), which yields one or more
         results.  Each result is either a tuple (lo, hi), if out_sig is a
         single index, or a tuple of such tuples, one for each index in out_sig 
 
+        gen_rng is a system-provided random number generator.
+
         Multiplexes the call over the broadcasted collection of components in
         in_sig.  Broadcasting occurs if an index is integer or length-1 integer
         list.
+
+        if `do_scalar` is True, additionally yield a scalar dimension, which
+        represents a rank-agnostic broadcasted dimension.
         """
         try:
             self._check_out_sig(out_sig)
@@ -1085,8 +1072,8 @@ class OpSchema(object):
             ind.dims_node_cls = ge.GenDims
 
         pri_idx = self._get_rank_idx(in_sig, out_sig)
-        gdims = ge.GenDims(out_sig, in_sig, func, pri_idx, do_scalar, max_prod,
-                pars)
+        gdims = ge.GenDims(self, out_sig, in_sig, func, pri_idx, do_scalar,
+                max_prod, pars)
         G.set_registry(self.dims_graph)
         parents = self._get_dims_nodes(in_sig)
         ranks_dnode = self._dims_node(ge.DimsInput, base.INDEX_RANKS)
@@ -1097,7 +1084,7 @@ class OpSchema(object):
         Generate dimensions of {out_idx} of appropriate rank such that the
         product is in [1, max_prod]
         """
-        def gen():
+        def gen(rng):
             yield 1, None
         try:
             self.gen_dims_func(out_idx, gen, '', max_prod, False)
@@ -1109,7 +1096,7 @@ class OpSchema(object):
         Generate dimensions of {out_idx} of appropriate rank such that the
         product is in [min_prod, max_prod]
         """
-        def gen():
+        def gen(rng):
             yield min_val, max_val
         try:
             self.gen_dims_func(out_idx, gen, '', int(1e10), False)
@@ -1303,7 +1290,7 @@ class OpSchema(object):
         """
         G.set_registry(self.gen_graph)
         pred_obj = pr.ArgInt(arg_name, lo, hi)
-        gen_obj = ge.Int(lo, hi)
+        gen_obj = ge.Int(self, lo, hi)
         schema = self._pred_node(pr.Schema)
         p_arg = P.add_node(pred_obj, schema)
         g_arg = G.add_node(gen_obj)
@@ -1653,7 +1640,7 @@ class OpSchema(object):
             inode = self.inf_graph[pri_idx]
             inode.func.add_shapes_constraint(cons)
 
-    def dims_pred(self, pred_name, pfunc, pfunc_t, indices, *extra):
+    def dims_pred(self, pred_name, pfunc, pfunc_t, indices, *arg_names):
         """
         Registers {pfunc} with the schema to be used as an additional
         predicate for {indexes} dimensions.
@@ -1665,26 +1652,21 @@ class OpSchema(object):
         {pred_name} is a name given to this custom predicate.  It may be used
         in error messages.
 
-        Called as pfunc(*index_shapes, *extra), where index_shapes
-        are the resolved shapes of {indices}.  
+        Called as pfunc(*index_shapes, *argvals), where index_shapes
+        are the resolved shapes of `indices` and argvals are the resolved
+        values of `arg_names`.
         """
-        if len(extra) > 0:
-            pfunc = Partial(pfunc, *extra)
-            pfunc_t = Partial(pfunc_t, *extra)
-
-        pred = base.IndexPredicate(pred_name, False, pfunc, pfunc_t, indices)
+        pred = base.IndexPredicate(pred_name, False, pfunc, pfunc_t, indices,
+                *arg_names)
         self.index_preds.append(pred)
 
-    def dims_pred_cw(self, pred_name, pfunc, pfunc_t, indices, *extra):
+    def dims_pred_cw(self, pred_name, pfunc, pfunc_t, indices, *arg_names):
         """
         Like dims_pred, but pfunc is called 'component-wise'.  That is, it is
         called once for each set of broadcasted index shapes.
         """
-        if len(extra) > 0:
-            pfunc = Partial(pfunc, *extra)
-            pfunc_t = Partial(pfunc_t, *extra)
-
-        pred = base.IndexPredicate(pred_name, True, pfunc, pfunc_t, indices)
+        pred = base.IndexPredicate(pred_name, True, pfunc, pfunc_t, indices,
+                *arg_names)
         self.index_preds.append(pred)
 
     def dims_pred_rng(self, idx, lo, hi):
@@ -1704,13 +1686,22 @@ class OpSchema(object):
         betw = lambda v, lo, hi: lo <= v <= hi
         betw_t = lambda v, lo, hi: f'{v} must be in [{lo}, {hi}]'
 
+        leq = Partial(leq, hi)
+        leq_t = Partial(leq_t, hi)
+
+        geq = Partial(geq, lo)
+        geq_t = Partial(geq_t, lo)
+
+        betw = Partial(betw, lo, hi)
+        betw_t = Partial(betw_t, lo, hi)
+
         if lo is None:
-            self.dims_pred_cw(f'{idx} >= {lo}', leq, leq_t, idx, hi)  
+            self.dims_pred_cw(f'{idx} >= {lo}', leq, leq_t, idx)  
         elif hi is None:
-            self.dims_pred_cw(f'{idx} >= {lo}', geq, geq_t, idx, lo)  
+            self.dims_pred_cw(f'{idx} >= {lo}', geq, geq_t, idx)  
         else:
             name = f'{idx} in [{lo}, {hi}]'
-            self.dims_pred_cw(name, betw, betw_t, idx, lo, hi) 
+            self.dims_pred_cw(name, betw, betw_t, idx) 
 
     def return_tensor(self, *sigs):
         """
@@ -1783,5 +1774,5 @@ class OpSchema(object):
         self._dot_graph(nodes, f'{stub}.inf')
 
         nodes = self.dims_graph.values()
-        self._dot_graph(nodes, f'{stub}.comp_dims')
+        self._dot_graph(nodes, f'{stub}.dims')
 
